/*
  Disappearable objects
 */

function DisappearableObject( element, endHandler, errorHandler ) {
    if ( endHandler == undefined ) {
        endHandler = function() {};
    }
    this.endHandler = endHandler;
    if ( errorHandler == undefined ) {
        errorHandler = __LOG;
    }
    this.errorHandler = errorHandler;
    if ( element == undefined ) {
        throw new Error( "no element" );
    }
    this.element = element;
    this.interval = 13;
    this.steps    = 6;
    this.counter  = 0;
    this.isRunning      = false;
    this.isVanishing    = false;
    this.isVanished     = false;
    this.initialOpacity = 1.0;
    this.targetOpacity  = 0.0;
    this.timer = null;

/*    this.elicit       = _DisappearableObject_elicit;*/
    this.transit      = _DisappearableObject_transit;
    this.setOpacity   = _DisappearableObject_setOpacity;
    this.addListener   = _DisappearableObject_addListener;
    this.removeListener   = _DisappearableObject_removeListener;
    this.__tick       = _DisappearableObject_tick;
    this.__getOpacity = _DisappearableObject_getOpacity;

    this.listener = null;
}

function  _DisappearableObject_setOpacity( op0, op1 ) {
    this.initialOpacity = op0;
    this.targetOpacity  = op1;
}

function _DisappearableObject_transit() {
    if ( this.isRunning ) return;
    this.isRunning = true;
    this.counter = 0;
    this.isVanishing = !this.isVanished;
    var self = this;
    this.timer = setInterval( function(){ self.__tick(); }, this.interval );
    this.__tick();
}

// function _DisappearableObject_vanh() {
//     if ( this.isRunning ) return;
//     this.isRunning = true;
//     this.counter = 0;
//     this.isVanishing = true;
// }

function _DisappearableObject_getOpacity() {
    if ( this.isVanished ) {
        return 1.0 - this.counter / this.steps;
    } else {
        return this.counter / this.steps;
    }
}

function _DisappearableObject_addListener( listener ) {
    this.listener = listener;
}
function _DisappearableObject_removeListener( listener ) {
    if ( this.listener == listener ) {
        this.listener = null;
    }
}

function _DisappearableObject_tick() {
    //showStatus( "COUNT " + this.counter + " => " + this.__getOpacity() + " " + this.isVanished );/// this.steps );
    this.counter ++;
    if ( this.listener != null ) {
        this.listener( this );
    }
    if ( this.counter > this.steps ) {
        var opacity;
        if ( this.isVanishing ) {
            opacity = this.targetOpacity;
        } else {
            opacity = this.initialOpacity;
        }
        this.element.style.opacity = opacity;
        this.isRunning   = false;
        this.isVanishing = false;
        if ( this.timer != null ) {
            clearInterval( this.timer );
            this.timer = null;
        }
        this.isVanished = !this.isVanished;
        //if ( this.endHandler ) {
        if ( this.endHandler != null ) this.endHandler( this );
    } else {
        var opacity = this.__getOpacity();
        this.element.style.opacity = opacity;
    }
}
